package client;

import java.net.*;
import java.io.*;

/**
 * The MessageReceiver is an own thread that runs continuously to receive
 * and display messages. The messages are received via a udp (datagram)
 * socket.
 *
 * Implement the marked part of the run()-methode below. Take a look
 * at the whole source code, especially at InstantMessenger and
 * NetworkUtilities. Use InstantMessenger.IM to have access to the only
 * existing instance of the InstantMessenger and the gui.
 */

public class MessageReceiver extends Thread {
	/** The UDP socket through which we receive the messages. */
	private DatagramSocket socket;

	public MessageReceiver(DatagramSocket socket) {
		super("IM MessageReceiver");

		this.socket = socket;
	}

	/**
	 * Run continuously to receive and display messages.
	 * Message are received in the specified format: type MESSAGE (1),
	 * messageLength (2), messageText (var).
	 */

	public void run() {
		byte[] message = new byte[65536];

		while (true) {
			try {
				/* get the actual packet */
				DatagramPacket packet = new DatagramPacket(message, message.length);
				socket.receive(packet); // blocking, until packet received!
				int packetLength = packet.getLength();
				if (packetLength < 3) { // minimum length is 3, type (1) + messageLength (2)
					System.out.println("Unrecognized protocol format.");
					continue; // Ignore corrupt packets
				}

				/* parse the message */
				byte type = message[0];
				if (type != NetworkUtilities.MESSAGE) {
					System.out.println("Unrecognized protocol message.");
					continue; // Ignore corrupt packets
				}
				int messageLength = NetworkUtilities.get16(message, 1);
				if (messageLength > packetLength - 3) { // Check for too long messages...
					messageLength = packetLength - 3;
				}
				String messageText = new String(message, 3, messageLength); // Create a readable message as String

				/* display it to the user */
				User user = InstantMessenger.IM.getUser(packet.getAddress().getAddress(), packet.getPort());
				String name;
				if (user == null) {
					name = "(unknown)";
					System.out.println("unknown is: " + packet.getAddress());
				} else {
					name = user.getName();
				}
				InstantMessenger.IM.displayMessage(name + ": " + messageText);
			} catch (IOException ioe) {
				// I/O error in socket
				System.out.println("Oops, we caused an IOError. Stopping.");
				ioe.printStackTrace();
				break; // Fatal error, socket/stream corrupt...
			} catch (Exception e) {
				// other exceptions during parsing
				System.out.println("Other problems. Waiting for new message.");
				e.printStackTrace();
				continue; // Ignore any other exception
			}
		}
		System.out.println("Message receiving client done.");
	}
}